/* (C) Northwestern University
 * See COPYING in the top-level directory . */

#define _XOPEN_SOURCE 500
#include <unistd.h>
#include <math.h>
#include "misc.h"

double generate_rand_zero_one(unsigned int *seed)
{
    /* Dividing an int by a double should return a double */
    return (rand_r(seed)) / ((double) (RAND_MAX*1.0));
}

int generate_int_range(unsigned int *seed,
		       int min,
		       int max)
{
    double rand_zero_one = generate_rand_zero_one(seed);
    double tmp_result = rand_zero_one * (max - min);

    assert(min <= max);
    
    if (tmp_result - floor(tmp_result) < ceil(tmp_result) - tmp_result)
	tmp_result = floor(tmp_result) + min;
    else
	tmp_result = ceil(tmp_result) + min;

    return ((int) tmp_result);
}
		     
int find_frag(unsigned int *seed,
	      int total_frags)
{
    int i;
    double rand_zero_one = generate_rand_zero_one(seed);

    for (i = 0; i < total_frags; i++)
    {
	if (rand_zero_one <= ((double) (i+1.0) / total_frags))
	{
	    return i;
	}
    }
    
    assert(i < total_frags);

    fprintf(stderr, "find_frag:Impossible for rand %f from 0 - 1\n",
	    rand_zero_one);
    return -1;
}
